package Linux::Disks;

use strict;
use warnings;

=encoding utf8

=head1 NAME

Linux::Disks -- find all blockdevices in a Linux System

=head1 VERSION

Version 0.01

=cut

our $VERSION = '0.01';

=head1 SYNOPSIS

 use Linux::Disks;
 my $disks=Linux::Disks->new();
 for my $dev ($disks->drives())
 {
   print " NAME        : ".$dev->name()."\n";
   print " DEVICE      : ".$dev->device()."\n";
   print " VENDOR/MODEL: ".$dev->vendor().'/'.$dev->model()."\n";
   print " MAJOR ID    : ".$dev->major()."\n";
   print " MINOR ID    : ".$dev->minor()."\n";
   print " BUS TYPE    : ".$dev->bus_type()."\n";
   print " UUID        : ".$dev->uuid()."\n";
   print " SERIAL      : ".$dev->serial()."\n";
   if($dev->mountable())
   {
     print " MOUNTED     : ".($dev->mounted()?'YES':'NO')."\n";
     print " IN FSTAB    : ".($dev->in_fstab()?'YES':'NO')."\n";
     print " FS TYPE     : ".$dev->fs_type()."\n";
     print " SPECIAL FS  : ".($dev->special_fs()?'YES':'NO')."\n";
     print " SIZE        : ".$dev->size()."\n";
     print " FREE        : ".$dev->free()."\n";
     print " MOUNT POINT : ".$dev->mount_point()."\n";
   }
   else
   {
     print " PARTITIONS  :\n";
     for my $part ($dev->partitions())
     {
       print "    NAME        : ".$part->name()."\n";
       print "    MAJOR ID    : ".$part->major()."\n";
       print "    MINOR ID    : ".$part->minor()."\n";
       print "    LABEL       : ".$part->label()."\n";
       print "    DEVICE      : ".$part->device()."\n";
       print "    IN FSTAB    : ".($part->in_fstab()?'YES':'NO')."\n";
       print "    MOUNTED     : ".($part->mounted()?'YES':'NO')."\n";
       print "    FS TYPE     : ".$part->fs_type()."\n";
       print "    SPECIAL FS  : ".($part->special_fs()?'YES':'NO')."\n";
       print "    SIZE        : ".$part->size()."\n";
       print "    FREE        : ".$part->free()."\n";
       print "    MOUNT POINT : ".$part->mount_point()."\n";
       print "    ---------------\n";
     }
   }
   print " -------------------\n";
 }

=head1 DESCRIPTION

This Modul uses System Tools (blkid,fdisk,df,udevadm)
and parse "/etc/fstab" and search in "/dev" to find all aviable Informations about Blockdevices.

=head1 AUTHOR

ToPeG

=head1 Linux::Disks

=head2 PACKAGE VARIABLES

=over 4

=cut

=item $dev_dir

Directory of the device filesystem

predefined: /dev

=item $blkid_cmd

Command to find all Blockdevice IDs

predefined: /sbin/blkid

=item $fdisk_cmd

Command for fdisk

predefined: LC_ALL=C /sbin/fdisk -l 2>/dev/null

=item $dfree_cmd

command for df

predefined: LC_ALL=C /bin/df -PTx tmpfs 2>/dev/null

=item $udev_cmd

command for udevadm

predefined: /sbin/udevadm info --query=all --name="%s" 2>/dev/null

=item $fstab_file

path to the fstab

predefined: /etc/fstab

=item $mount_dir

path to the default mount directory

predefined: /media/

path to the mountpoint of sysfs

=item $sys_dir

predefined: /sys/

=cut

our $dev_dir='/dev/';
our $blkid_cmd='/sbin/blkid';
our $fdisk_cmd='LC_ALL=C /sbin/fdisk -l 2>/dev/null';
our $dfree_cmd='LC_ALL=C /bin/df -PTx tmpfs 2>/dev/null';
our $udev_cmd='/sbin/udevadm info --query=all --name="%s" 2>/dev/null';
our $fstab_file='/etc/fstab';
our $mount_dir='/media/';
our $sys_dir='/sys/';

# Test needed programms
sub __test_prgs
{
  my $pos=shift;
  my @prgs=qw(/sbin/blkid /sbin/fdisk /bin/df /sbin/udevadm);
  my @vers=([2,20,1],     [1,2,4],    [8,13], [175]);
  return ('not in list',@prgs+0) if(!defined($pos) || $pos < 0 || $pos > $#prgs);
  my $path=$prgs[$pos];
  return ('not found',$path) unless(-f $path);
  return ('not executable',$path) unless(-x $path);
  my $add=' --version 2>&1';
  open(my $ph, '-|', $path.$add) or return (0,$path,'');
  my @version;
  while(<$ph>)
  {
    if($_=~/(\d+(?:\.\d+)*)/)
    {
      @version=split(/\./,$1);
      last;
    }
  }
  close($ph);
  return ('no version',$path) unless(@version);
  while(@{$vers[$pos]})
  {
    my $v1=shift(@version) // 0;
    my $v2=shift(@{$vers[$pos]});
    return ('wrong version',$path) if($v1<$v2);
  }
  return ('ok',$path);
}

# Test needed Files
sub __test_files
{
  my $pos=shift;

  my $dd=$dev_dir.'/';
  $dd=~s!/+!/!gs;

  my @files=( [$dd.'hd*', $dd.'sd*', $dd.'md*'], [$fstab_file]);

  return ('not in list',@files+0) if(!defined($pos) || $pos < 0 || $pos > $#files);

  my @paths;
  push(@paths,glob($_)) for(@{$files[$pos]});

  return ('not found',join(', ',@{$files[$pos]})) unless(@paths);

  my $ok=0;
  ( -f $_ or -b _ ) and $ok++ for(@paths);
  return ('not found',join(', ',@{$files[$pos]})) unless($ok);

  return ('ok', join(', ',@{$files[$pos]}));
}

=back

=head2 METHODS

=over 4

=item new(%OPTIONS)

create a new Object

Options

fake : a hashtree of deviceinformations. See result and code of __parse for more. Only for testpurpes

=cut
sub new
{
  my $class=shift;
  pop(@_) if(@_%2);
  my %opts=@_;
  my $self={};
  $self->{opts}=\%opts;
  $self->{err}=[];
  $self->{devs}={};
  if($opts{fake})
  {
    #print Dumper($opts{fake});
    $self->{devs}=$opts{fake};
  }
  else
  { __parse($self->{devs},$self->{err}); }

  bless($self, ref($class) || $class );
}

=item update()

search for new Disks and Partitions.
Remove old  Disk-Informations

=cut
sub update
{
  my $self=shift;
  __parse($self->{devs},$self->{err});
}

=item update_partitions()

actualisize the partition information

it will not serach for new Disks or Partitions

=cut
sub update_partitions
{
  my $self=shift;
  __parse_blkid($self->{devs},$self->{err});
  __parse_diskfree($self->{devs},$self->{err});
}

=item drives()

Retuns a list of Linux::Disks::Drive Objects

=cut
sub drives
{
  my $self=shift;
  my @l;
  for my $dev (sort(keys(%{$self->{devs}})))
  { push(@l,Linux::Disks::Drive->_new($self->{devs}->{$dev})); }
  return @l;
}
=item partitions()

Returns a list of Linux::Disks::Partition Objects

=cut
sub partitions
{
  my $self=shift;
  my @l;
  for my $dev (sort(keys(%{$self->{devs}})))
  {
    my $parts=$self->{devs}->{$dev}->{part};
    for my $part (sort(keys(%$parts)))
    {
      push(@l,Linux::Disks::Partition->_new($parts->{$part}));
    }
  }
  return @l;
}

=item has_errors()

returs 1 if an Error occured

=cut
sub has_errors
{
  my $self=shift;
  return 1 if(@{$self->{err}});
  return 0;
}

=item errors()

return a list of all Errors.

Resets the internel Errorlist.

=cut
sub errors
{
  my $self=shift;
  if(@{$self->{err}})
  {
    my @r=@{$self->{err}};
    @{$self->{err}}=();
    return @r;
  }
  return ();
}


#-----------------------------------------------------------------------
sub __parse
{
  my $disks=shift() // {};
  my $error=shift() // [];
  my $cmd1=shift()  // $fdisk_cmd;
  my $cmd2=shift()  // $dfree_cmd;
  my $cmd3=shift()  // $udev_cmd;
  my $cmd4=shift()  // $blkid_cmd;
  my $cmd5=shift()  // $fstab_file;

  %$disks=();

  for my $part (glob($dev_dir.'hd*'),glob($dev_dir.'sd*'),glob($dev_dir.'md*'))
  {
    my $disk=$part;
    $disk=~s/\d+$//;
    my $k=\$disks->{$disk};
    if($disk ne $part)
    {
      $k=\$disks->{$disk}->{part}->{$part};
      $disk=$part;
    }
    $$k->{name} = $disk;
    $$k->{udev} = __parse_udev($disk,$error,$cmd3);
  }

  #print Dumper($disks); exit;

  if(open(my $ch, '-|', $cmd1))
  {
    my $device=undef;
    while(my $line=<$ch>)
    {
      if($line=~/Disk ([^:]+).+?, (\d+) bytes/)
      {
        my $name=$1;
        my $disk=$1;
        $disk=~s/\d+$//s;
        my $size=$2;

        my $k=\$disks->{$disk};

        if($name ne $disk)
        { $k=\$disks->{$disk}->{part}->{$name}; }

        $$k->{name} = $name;
        $$k->{size} = $size;
      }
      elsif($device && $line=~m!^(\Q$device->{name}\E\d+)\s+(\*?)\s+(\d+)\s+(\d+)\s+(\d+)\s+\d+\s+(.+?)\s+$!s)
      {
        my $name=$1;
        my $boot=0;
        $boot=1 if($2);
        my $k=\$device->{part}->{$1};
        $$k->{name}      =$1,
        $$k->{boot}      =$boot,
        $$k->{start}     =$3,
        $$k->{end}       =$4,
        $$k->{allblocks} =$5,
        $$k->{type}      =$6,
      }
    }
    close($ch) || push(@$error, qq(Can't run "$cmd1" ($!)) );
  }
  else
  { push(@$error, qq(Can't run "$cmd1" ($!)) ); }
  __parse_fstab($disks,$error,$cmd5);
  __parse_blkid($disks,$error,$cmd4);
  __parse_diskfree($disks,$error,$cmd2);

  return $disks;
}

sub __parse_fstab
{
  my $device=shift;
  my $errors=shift // [];
  my $file=shift // $fstab_file;

  # Read the fstab
  local $/ = "\n";
  if ( open(my $fh, '<', $file) )
  {
    while (<$fh>)
    {
      next if ( /^\s*#/ || /^\s*$/ );

      $_=~s/^\s+//;
      my @vals = split(/\s+/, $_ );

      $vals[0]=~s!/+$!!;

      my $name=$vals[0];

      if($name=~/^UUID=(.+)$/)
      { $name="${dev_dir}disk/by-uuid/$1"; }

      my $udev=__parse_udev($name,$errors);

      if(exists($udev->{DEVNAME}))
      { $name=$udev->{DEVNAME}; }

      my $dev=$name;
      my $type=$vals[2];
      if($type eq 'cifs')
      { ($dev,undef)=__split_samba($dev); }
      elsif($type eq 'nfs')
      { ($dev,undef)=__split_nfs($dev); }
      elsif($type eq 'afs')
      { ($dev,undef)=__split_afs($dev); }
      else
      { $dev=~s/\d+$//s; }

      my $k=\$device->{$dev};
      if($name ne $dev)
      {
        $$k->{name}=$dev unless($$k->{name});
        $$k->{mountable}=0;
        $k=\$device->{$dev}->{part}->{$name};
      }

      #print "FSTAB: $dev => $name\n";

      $$k->{name}      = $name;
      $$k->{mpoint}    = $vals[1];
      $$k->{type}      = $vals[2];
      $$k->{mopts}     = $vals[3] // '';
      $$k->{udev}      = $udev unless($$k->{udev});
      $$k->{mountable} = 1;
      $$k->{fstab}     = 1;

    }
    close($fh) || push(@$errors,"Can't open $file ($!)");
  }
  else
  { push(@$errors,"Can't open $file ($!)"); }
}

sub __parse_udev
{
  my $device=shift;
  my $errors=shift // [];
  my $cmd= shift // $udev_cmd;

  return {} unless($device && -b $device);

  my %data;
  $cmd=sprintf($cmd,$device);
  if(open(my $ph, '-|', $cmd))
  {
    while(my $line=<$ph>)
    {
      if($line=~/^E:\s+(\w+)=(.+)$/)
      {
        my $name=$1;
        my $vals=$2;
        if($vals=~/ /)
        { $vals=[map{$_=~s/\\x(\d\d)/pack('H*',$1)/gse;$_}split(/ /,$vals)] }
        else
        { $vals=~s/\\x(\d\d)/pack('H*',$1)/gse; }

        if($name eq 'DEVLINKS' && ref($vals) ne 'ARRAY')
        { $vals=[$vals]; }

        $data{$name}=$vals;
      }
    }
    close($ph) || push(@$errors,"Can't run $cmd ($!)");
  }
  else
  {push(@$errors,"Can't run $cmd ($!)");}
  return \%data;
}

sub __parse_blkid
{
  my $disks=shift;
  my $error=shift // [];
  my $cmd=shift() // $blkid_cmd;

  my $type='';

  if(open(my $ch, '-|', $cmd))
  {
    while( my $line=<$ch>)
    {
      chomp($line);
      if($line=~/^([^:]+?)\:\s+((?:\w+="[^"]+"\s*)+)/)
      {
        my $name=$1;
        my $dev=$1;
        my $vals=$2;

        $dev=~s/\d+$//;
        unless($disks->{$dev}->{name})
        { $disks->{$dev}->{name}=$dev; }
        my $k=\$disks->{$dev}->{part}->{$name};

        $$k->{udev}=__parse_udev($name,$error) unless($$k->{udev});

        while($vals=~s!(\w+)="([^"]+)"!!)
        { $$k->{blkid}->{$1}=$2; }

        $$k->{mountable} = 1;
      }
    }
    close($ch) or push(@$error, qq(Can't run "$cmd" ($!)) );
  }
  else
  { push(@$error, qq(Can't run "$cmd" ($!)) ); }
}

sub __parse_diskfree
{
  my $disks=shift;
  my $error=shift // [];
  my $cmd2=shift() // $dfree_cmd;

  for my $dsk (values(%$disks))
  {
    $dsk->{mounted}=0;
    for my $dev (values(%{$dsk->{part}}))
    { $dev->{mounted}=0; }
  }

  if(open(my $ch, '-|', $cmd2))
  {
    <$ch>;
    while(my $line=<$ch>)
    {
      if( $line=~m!^(.+?)\s+(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)%\s+(.+?)\s*$!s )
      {
        my @l=($1,$2,$3,$4,$5,$6,$7);

        $l[0]=~s!/+$!!;

        my $dev=$l[0];
        my $type=$l[1];

        # link to device
        if(-l $dev)
        {
          my @dd=split('/',readlink($dev));
          if($dd[0] eq '..')
          {
            my @d=split('/',$dev);
            pop(@d);
            while($dd[0] eq '..')
            {
              pop(@d);
              shift(@dd);
            }
            $dev=join('/',@d,@dd);
          }
          else
          { $dev=join('/',@dd); }
          $l[0]=$dev;
        }

        if($type eq 'cifs')
        { ($dev,undef)=__split_samba($dev); }
        elsif($type eq 'nfs')
        { ($dev,undef)=__split_nfs($dev); }
        elsif($type eq 'afs')
        { ($dev,undef)=__split_afs($dev); }
        else
        { $dev=~s/\d+$//s; }
        my $k=\$disks->{$dev};
        if($l[0] ne $dev)
        {
          $$k->{name}=$dev unless($$k->{name});
          $$k->{mountable}=0;
          $k=\$$k->{part}->{$l[0]};
        }
        @{$$k}{qw(name type blocks used free percent mpoint)}=@l;
        $$k->{mounted}=1;
        $$k->{mountable}=1;
      }
    }
    close($ch) or push(@$error, qq(Can't run "$cmd2" ($!)) );
  }
  else
  { push(@$error, qq(Can't run "$cmd2" ($!)) ); }
}

#-----------------------------------------------------------------------

sub __split_samba
{
  my $name=shift;
  my @l=split(/\//,$name);
  shift(@l) while(!$l[0] && @l);
  my $disk='//'.shift(@l);
  pop(@l) while(@l && !$l[-1]);
  my $part=join('/',@l);
  return ($disk,$part);
}

sub __split_nfs
{
  my $name=shift;
  return split(/:/,$name);
}

sub __split_afs
{
  my $name=shift;
  if($name=~m!^\s*afp://((?:[^:]+:[^@]+@)?[^/]+)(.*)$!)
  { return ($1,$2); }
  return ($name,'???');
}
=back
=cut
########################################################################
########################################################################
package Linux::Disks::Partition;

=head1 Linux::Disks::Partition

=head2 METHODS

=over 4

=cut
sub _new
{
  my $class=shift;
  my $ref=shift;
  my $self={data=>$ref};
  bless($self,ref($class) || $class);
}

=item size()

returns the size of the partition in bytes

=cut
sub size
{
  my $self=shift;

  if(exists($self->{data}->{blocks}))
  { return $self->{data}->{blocks}*1024; }

  if(exists($self->{data}->{allblocks}))
  { return $self->{data}->{allblocks}*1024; }

  if(exists($self->{data}->{size}))
  { return $self->{data}->{size} // 0; }

  if(exists($self->{data}->{udev}->{UDISKS_PARTITION_SIZE}))
  { return $self->{data}->{udev}->{UDISKS_PARTITION_SIZE}; }

  return 0;
}

=item used()

if aviable it's returns the bytes used
0 otherwise

=cut
sub used
{
  my $self=shift;
  my $z=$self->{data}->{used} // 0;
  $z*=1024;
  return $z;
}

=item free()

returns the free Partitionspace if aviable or 0

=cut
sub free
{
  my $self=shift;
  my $z=$self->{data}->{free} // 0;
  $z*=1024;
  return $z;
}

=item percent()

returns the percent usage of the Partition if aviable or 0

=cut
sub percent
{
  my $self=shift;
  my $z=0;
  if(exists($self->{data}->{percent}))
  { $z=$self->{data}->{percent}; }
  elsif($self->size())
  { $z=int((100/$self->size())*$self->used()); }
  return $z;
}

=item bootable()

return 1 if the partition have the bootflag

=cut
sub bootable
{
  my $self=shift;
  return $self->{data}->{boot} // 0;
}

=item mount_point()

returns a path to the mountpoint.

A mountpoint will be generated if not set

=cut
sub mount_point
{
  my $self=shift;

  $self->_set_mount_point(@_) if(@_);

  if(defined($self->{data}->{mpoint}) && $self->{data}->{mpoint})
  { return $self->{data}->{mpoint}; }

  my $name=$Linux::Disks::mount_dir.'/'.$self->label();
  $name=~s!/+!/!gs;
  $self->{data}->{mpoint}=$name;
  return $name;
}

sub _set_mount_point
{
  my $self=shift;
  my $new=shift;
  return 0 unless($new);

  my $point=$self->mount_point();

  my $mdir=$Linux::Disks::mount_dir;
  $mdir=~s!/+$!!;

  return 0 unless($point=~m!^\Q$mdir\E!);
  return 0 unless($new=~m!^\Q$mdir\E!);


  $self->{data}->{mpoint}=$new;
  return 1;
}

=item start()

returns the Startposition of the Partition

=cut
sub start
{
  my $self=shift;
  return $self->{data}->{start} // 0;
}

=item end()

the endposition of the Partition

=cut
sub end
{
  my $self=shift;
  return $self->{data}->{end} // 0;
}

=item fs_type()

the Filesystem Type

=cut
sub fs_type
{
  my $self=shift;

  if(exists($self->{data}->{udev}->{ID_FS_TYPE}))
  { return $self->{data}->{udev}->{ID_FS_TYPE}; }

  if(exists($self->{data}->{blkid}->{TYPE}))
  { return $self->{data}->{blkid}->{TYPE}; }

  if($self->{data}->{type})
  { return $self->{data}->{type} };

  return '';
}

=item special_fs()

1 if it's an special fs.

Detects: raid, swap, usbfs, proc, rootfs, tmpfs

=cut
sub special_fs
{
  my $self=shift;
  my $fs=$self->fs_type();
  return 1 unless($fs);
  return 1 if($fs =~ /raid/);
  return 1 if($fs eq 'swap');
  return 1 if($fs eq 'usbfs');
  return 1 if($fs eq 'proc');
  return 1 if($fs eq 'rootfs');
  return 1 if($fs =~/tmpfs/);
  return 0;
}


=item uuid()

the UUID of the Partition

=cut
sub uuid
{
  my $self=shift;
  if(exists($self->{data}->{udev}->{ID_FS_UUID_ENC}))
  { return $self->{data}->{udev}->{ID_FS_UUID_ENC}; }

  if(exists($self->{data}->{blkid}->{UUID}))
  { return $self->{data}->{blkid}->{UUID}; }

  if(exists($self->{data}->{part}))
  {
    my $p=(values(%{$self->{data}->{part}}))[0];
    if(exists($p->{udev}->{ID_FS_UUID_ENC}))
    { return $p->{udev}->{ID_FS_UUID_ENC}; }

    if(exists($p->{blkid}->{UUID}))
    { return $p->{blkid}->{UUID}; }
  }

  return '';
}

=item mounted()

1 if the Partition is mounted

=cut
sub mounted
{
  my $self=shift;
  return $self->{data}->{mounted};
}

=item mountable()

1 if the Partition is mountable.

A Partition is mountable if the Bockdevice is avaiable,
the mountpoints exists and is and Directory
and the fstype is aviable

=cut
sub mountable
{
  my $self=shift;
  my $ok=$self->{data}->{mountable}?1:0;

  unless($ok)
  {
    if(-b $self->name() && -d $self->mount_point() && $self->fs_type())
    {
      $self->{data}->{mountable}=1;
      $ok=1;
    }
  }

 return $ok;
}

=item in_fstab()

1 if the Partition is mentiond in the fstab.

=cut
sub in_fstab { return shift->{data}->{fstab}?1:0; }


=item name()

name of the Partition

=cut
sub name
{
  my $self=shift;
  return $self->{data}->{name} // '';
}

=item device()

device of the Partition

=cut
sub device
{
  my $self=shift;
  my $device='none';
  if(exists($self->{data}->{udev}->{DEVNAME}))
  { $device=$self->{data}->{udev}->{DEVNAME}; }
  return $device;
}

=item links()

alternative devicenames

=cut
sub links
{
  my $self=shift;
  my @l;
  if($self->{data}->{udev}->{DEVLINKS})
  {
    if(ref($self->{data}->{udev}->{DEVLINKS}))
    { @l=@{$self->{data}->{udev}->{DEVLINKS}}; }
    else
    { @l=($self->{data}->{udev}->{DEVLINKS}); }
  }
  return @l;
}

=item path()

sys-path of the Partition

=cut
sub path
{
  my $self=shift;
  my $path=$self->_udev_get('DEVPATH');
  if($path)
  {
    my $ret=$Linux::Disks::sys_dir.'/'.$path;
    $ret=~s!/+!/!gs;
    return  $ret;
  }
  return '';
}

=item label()

Label of the Partition

=cut
sub label
{
  my $self=shift;

  my $label=$self->_udev_get('ID_FS_LABEL');
  return $label if($label);

  if(exists($self->{data}->{blkid}->{LABEL}))
  { return $self->{data}->{blkid}->{LABEL}; }

  if(defined($self->{data}->{mpoint}))
  {
    my $p=$self->{data}->{mpoint};
    $p=~s!^.+/!!s;
    return $p;
  }

  if($self->uuid())
  { return $self->uuid(); }

  #                                prevent call loop
  #                          ______________/\____________
  #                         /                            \
  return $self->bus_type.'_'.$self->_udev_get('ID_VENDOR').'_'.$self->model().'_'.$self->serial();
}

=item major()

Major Device Name

=cut
sub major{ return $_[0]->_udev_get('MAJOR'); }

=item minor()

Minor Device Name

=cut
sub minor{ return $_[0]->_udev_get('MINOR'); }

=item vendor()

Vendor Device Name

=cut
sub vendor
{
  my $self=shift;
  my $m=$self->_udev_get('ID_VENDOR');
  unless($m)
  {
    $m=$self->name();
    my @l = split(m!/+!,$m);
    while(@l!=0 && $l[0] eq '')
    { shift(@l); }
    $m=$l[0] if(@l);
  }
  return $m;
}

=item model()

Device Model

=cut
sub model {
  my $self=shift;
  my $m=$self->_udev_get('ID_MODEL');
  unless($m)
  {
    $m=uc($self->bus_type().'::'.$self->fs_type());
    $m=~s/^:+//;
    $m=~s/:+$//;
  }
  return $m;
}

=item serial_shor()

short serial number of the device

=cut
sub serial_short { return $_[0]->_udev_get('ID_SERIAL_SHORT'); }

=item serial()

serial number of the Device

=cut
sub serial
{
  my $self=shift;
  my $serial=$_[0]->{data}->{udev}->{ID_SERIAL} // '';
  return $serial if($serial);
  return $self->serial_short();
}

=item revision()

revision of the Devide

=cut
sub revision { return $_[0]->_udev_get('ID_REVISION'); }

=item bus_type()

bustype of the Device

Aviable: scsi, usb, ide, net, raid

=cut
sub bus_type
{
  my $self=shift;
  my $type = $self->_udev_get('ID_BUS');
  if(!$type)# && $self->mountable())
  {
    if($self->name()=~m!/md!)
    { $type='raid'; }
    else
    {
      my $b=$self->fs_type();
      if(!$b && $self->can('partitions'))
      {
        my @p=$self->partitions();
        $b=$p[0]->fs_type() if(@p);
      }
      $b='' unless($b);
      my @types=qw(nfs smbfs cifs afs);
      if(grep{$b eq $_}@types)
      { $type='net'; }
    }
  }
  return $type;
}


sub _udev_get
{
  my $self=shift;
  my $name=shift;
  if($self->{data}->{udev}->{$name})
  { return $self->{data}->{udev}->{$name}; }

  if(exists($self->{data}->{part}))
  {
    my $p=(values(%{$self->{data}->{part}}))[0];
    if($p->{udev}->{$name})
    { return $p->{udev}->{$name}; }
  }

  return '';
}


=back
=cut
########################################################################
########################################################################
package Linux::Disks::Drive;
use base 'Linux::Disks::Partition';

=head1 Linux::Disks::Drive

=head2 METHODS

=over 4

=item size()

returns the size of the Device in bytes

=item used()

if aviable it's returns the bytes used
0 otherwise

=item free()

returns the free Diskspace if aviable or 0

=item percent()

returns the percent usage of the Partition if aviable or 0

=item bootable()

return 0

=item mount_point()

returns a path to the mountpoint.

A mountpoint will be generated if not set

=item start()

returns the Startposition of the Disk

=item end()

the endposition of the Disk

=item fs_type()

the Filesystem Type

=item special_fs()

1 if it's an special fs.

Detects: raid, swap, usbfs, proc, rootfs, tmpfs

=item uuid()

the UUID of the Partition

=item mounted()

1 if the Partition is mounted

=item mountable()

1 if the Device is mountable.

A Partition is mountable if the Bockdevice is avaiable,
the mountpoints exists and is and Directory
and the fstype is aviable

=item in_fstab()

1 if the Partition is mentiond in the fstab.

=item name()

name of the Partition

=item device()

device of the Partition

=item links()

alternative devicenames

=item path()

sys-path of the Partition

=item label()

Label of the Partition

=item major()

Major Device Name

=item minor()

Minor Device Name

=item vendor()

Vendor Device Name

=item model()

Device Model

=item serial_shor()

short serial number of the device

=item serial()

serial number of the Device

=item revision()

revision of the Devide

=item bus_type()

bustype of the Device

Aviable: scsi, usb, ide, net, raid

=cut

sub _new
{
  my $class=shift;
  my $ref=shift;
  my $self={data=>$ref};
  bless($self,ref($class) || $class);
}

=item partitions()

returns a list of all Partitions on this Device

=cut
sub partitions
{
  my $self=shift;
  my @l;
  for my $part (sort(keys(%{$self->{data}->{part}})))
  {
    push(@l,Linux::Disks::Partition->_new( $self->{data}->{part}->{$part} ));
  }
  return @l;
}

=back

=head1 AUTHOR

Tobias Grönhagen <linux_disks@topeg.de>

=head1 BUGS

Please report any bugs or feature requests to C<bug-linux-disks at rt.cpan.org>, or through
the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Linux-Disks>.  I will be notified, and then you'll
automatically be notified of progress on your bug as I make changes.





=head1 LICENSE AND COPYRIGHT

Copyright 2012 Tobias Grönhagen.

This program is free software; you can redistribute it and/or modify it
under the terms of either: the GNU General Public License as published
by the Free Software Foundation; or the Artistic License.

See http://dev.perl.org/licenses/ for more information.

=cut

1;